﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MathTools
{
    /// <summary>
    /// Аналог класса Math библиотеки FCL
    /// </summary>
    public class MyMath
    {
        //Константы класса
        const double TWOPI = 2 * Math.PI;
        const double EPS = 1E-10;

        //Статические методы класса

        /// <summary>
        /// Sin(x)
        /// </summary>
        /// <param name="x">
        ///     угол в радианах - аргумент функции Sin
        /// </param>
        /// <returns>
        ///     Возвращает значение функции Sin для заданного угла
        /// </returns>
        public static double Sin(double x)
        {
            //Оптимизация - приведение к интервалу
            x = x % TWOPI;

            //Init
            double a = x;
            double res = 0;
            int k = 0;
            double x2 = x * x;

            //Основные вычисления
            while (Math.Abs(a) > EPS)
            {
                res += a;
                k+=2;
                a *= -x2 / (k * (k + 1));                
            }
            return res;
        }

        /// <summary>
        /// Cos(x)
        /// </summary>
        /// <param name="x">
        ///     угол в радианах - аргумент функции Cos
        /// </param>
        /// <returns>
        ///     Возвращает значение функции Cos для заданного угла
        /// </returns>
        public static double Cos(double x)
        {
            //Оптимизация - приведение к интервалу
            x = x % TWOPI;

            //Init
            double a = 1;
            double res = 0;
            int k = 0;
            double x2 = x * x;

            //Основные вычисления
            while (Math.Abs(a) > EPS)
            {
                res += a;
                k += 2;
                a *= -x2 / (k * (k - 1));
            }
            return res;
        }
        /// <summary>
        /// Ctg(x)
        /// </summary>
        /// <param name="x">
        ///     угол в радианах - аргумент функции Ctg
        /// </param>
        /// <returns>
        ///     Возвращает значение функции Ctg для заданного угла
        /// </returns>
        public static double Ctg(double x)
        {
            //Оптимизация - приведение к интервалу
            x = x % Math.PI;
            return Cos(x) / Sin(x);            
        }
        /// <summary>
        /// Ctg(x)
        /// </summary>
        /// <param name="x">
        ///     угол в радианах - аргумент функции Ctg
        /// </param>
        /// <returns>
        ///     Возвращает значение функции Ctg для заданного угла
        /// </returns>
        public static double CtgV(double x)
        {
            //Оптимизация - приведение к интервалу
            x = x % Math.PI;
            double c = x;
            double b = x * x;
            double d = 1.0 / 3;
            double a = c * d;
            double S = 0;
            int i =1;
            while (Math.Abs(a) > EPS)
            {
                S += a;
                i++;
                d = D(i);
                c *= b;
                a = c * d;

            }            

            return (1 / x) - S;
        }
        static double D(int i)
        {
            int j = i << 1;
            int k = 1;
            double a = 0;
            double S = 0;
            while ((a = 1 / Math.Pow(k, j)) > EPS)
            {
                S += a;
                k++;
            }
            return 2*S/Math.Pow(Math.PI,j);
        }

        /// <summary>
        /// Sin(x) - без дополнительной оптимизации  
        /// </summary>
        /// <param name="x">
        ///     угол в радианах - аргумент функции Sin
        /// </param>
        /// <returns>
        ///     Возвращает значение функции Sin для заданного угла
        /// </returns>
        public static double SinOld(double x)
        {
            //Оптимизация - приведение к интервалу
            x = x % TWOPI;

            //Init
            double a = x;
            double res = 0;
            int k = 0;

            //Основные вычисления
            while (Math.Abs(a) > EPS)
            {
                res += a;
                a *= -x * x / ((2 * k + 2) * (2 * k + 3));
                k++;
            }
            return res;
        }
        public static double Test1(double x)
        {
            return x;
        }
        public static double Test2(double x)
        {
            //Init
            double a = x;
            double res = 0;

            //Основные вычисления
            while (Math.Abs(a) > EPS)
            {
                res += a;
                a *= 1e-3;               
            }
            return res;

        }


    }
}
